import uwsgi
import unittest
import random
import string


class BitmapTest(unittest.TestCase):

    __caches__ = [
        'items_1',
        'items_2',
        'items_3',
        'items_4',
        'items_17',
        'items_4_10',
        'items_1_100000',
        'items_non_bitmap',
        'items_lru'
    ]

    def setUp(self):
        for cache in self.__caches__:
            uwsgi.cache_clear(cache)

    def test_failed_by_one(self):
        self.assertIsNone(uwsgi.cache_update('key1', 'HELLO', 0, 'items_1'))

    def test_ok_four_bytes(self):
        self.assertTrue(uwsgi.cache_update('key1', 'HELL', 0, 'items_1'))

    def test_two_items_using_four_blocks(self):
        self.assertTrue(uwsgi.cache_update('key1', 'HE', 0, 'items_2'))
        self.assertTrue(uwsgi.cache_update('key2', 'LL', 0, 'items_2'))
        self.assertTrue(uwsgi.cache_del('key1', 'items_2'))
        self.assertIsNone(uwsgi.cache_update('key1', 'HEL', 0, 'items_2'))
        self.assertTrue(uwsgi.cache_update('key1', 'HE', 0, 'items_2'))

    def test_overlapping(self):
        self.assertTrue(uwsgi.cache_update('key1', 'HE', 0, 'items_2'))
        self.assertIsNone(uwsgi.cache_update('key1', 'HELL', 0, 'items_2'))
        self.assertTrue(uwsgi.cache_del('key1', 'items_2'))
        self.assertTrue(uwsgi.cache_update('key1', 'HELL', 0, 'items_2'))

    def test_big_item(self):
        self.assertIsNone(uwsgi.cache_update('key1', 'HELLOHELLOHELLOHEL', 0, 'items_17'))
        self.assertTrue(uwsgi.cache_update('key1', 'HELLOHELLOHELLOHE', 0, 'items_17'))

    def test_set(self):
        self.assertTrue(uwsgi.cache_set('key1', 'HELLO', 0, 'items_17'))
        self.assertIsNone(uwsgi.cache_set('key1', 'HELLO', 0, 'items_17'))
        self.assertTrue(uwsgi.cache_del('key1', 'items_17'))
        self.assertTrue(uwsgi.cache_set('key1', 'HELLO', 0, 'items_17'))
        self.assertIsNone(uwsgi.cache_set('key1', 'HELLO', 0, 'items_17'))

    def test_too_much_items(self):
        self.assertTrue(uwsgi.cache_set('key1', 'HELLO', 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_set('key2', 'HELLO', 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_set('key3', 'HELLO', 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_set('key4', 'HELLO', 0, 'items_4_10'))
        self.assertIsNone(uwsgi.cache_set('key5', 'HELLO', 0, 'items_4_10'))

    def test_big_delete(self):
        self.assertTrue(uwsgi.cache_set('key1', 'X' * 50, 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_del('key1', 'items_4_10'))
        self.assertTrue(uwsgi.cache_set('key1', 'HELLOHELLO', 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_set('key2', 'HELLOHELLO', 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_set('key3', 'HELLOHELLO', 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_set('key4', 'HELLOHELLO', 0, 'items_4_10'))
        self.assertIsNone(uwsgi.cache_set('key5', 'HELLOHELLO', 0, 'items_4_10'))

    def test_big_update(self):
        self.assertTrue(uwsgi.cache_set('key1', 'X' * 40, 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_update('key1', 'X' * 10, 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_del('key1', 'items_4_10'))
        self.assertIsNone(uwsgi.cache_update('key1', 'X' * 51, 0, 'items_4_10'))
        self.assertTrue(uwsgi.cache_update('key1', 'X' * 50, 0, 'items_4_10'))

    def test_multi_clear(self):
        for i in range(0, 100):
            self.assertTrue(uwsgi.cache_clear('items_4_10'))

    def test_multi_delete(self):
        for i in range(0, 100):
            self.assertTrue(uwsgi.cache_set('key1', 'X' * 50, 0, 'items_4_10'))
            self.assertTrue(uwsgi.cache_del('key1', 'items_4_10'))

        for i in range(0, 100):
            self.assertIsNone(uwsgi.cache_set('key1', 'X' * 51, 0, 'items_4_10'))
            self.assertIsNone(uwsgi.cache_del('key1', 'items_4_10'))

        for i in range(0, 100):
            self.assertTrue(uwsgi.cache_set('key1', 'X' * 50, 0, 'items_4_10'))
            self.assertTrue(uwsgi.cache_del('key1', 'items_4_10'))

    def test_big_key(self):
        self.assertTrue(uwsgi.cache_set('K' * 2048, 'X' * 50, 0, 'items_4_10'))
        self.assertIsNone(uwsgi.cache_set('K' * 2049, 'X' * 50, 0, 'items_4_10'))

    def rand_blob(self, n=32):
        return ''.join(random.choice(string.ascii_letters + string.digits) for x in range(n))

    def test_big_random(self):
        blob = self.rand_blob(100000)
        self.assertTrue(uwsgi.cache_set('KEY', blob, 0, 'items_1_100000'))
        get_blob = uwsgi.cache_get('KEY', 'items_1_100000')
        self.assertEqual(blob, get_blob)
        self.assertTrue(uwsgi.cache_del('KEY', 'items_1_100000'))
        self.assertIsNone(uwsgi.cache_set('KEY', 'X' * 100001, 0, 'items_1_100000'))
        self.assertTrue(uwsgi.cache_set('KEY', 'X' * 10000, 0, 'items_1_100000'))

    def test_non_bitmap(self):
        self.assertTrue(uwsgi.cache_set('KEY', 'X' * 20, 0, 'items_non_bitmap'))
        self.assertTrue(uwsgi.cache_del('KEY', 'items_non_bitmap'))
        self.assertIsNone(uwsgi.cache_set('KEY', 'X' * 21, 0, 'items_non_bitmap'))
        self.assertTrue(uwsgi.cache_set('KEY', 'X' * 20, 0, 'items_non_bitmap'))

    def test_lru(self):
        self.assertTrue(uwsgi.cache_set('KEY1', 'X' * 20, 0, 'items_lru'))
        self.assertTrue(uwsgi.cache_set('KEY2', 'X' * 20, 0, 'items_lru'))
        self.assertTrue(uwsgi.cache_set('KEY3', 'Y' * 20, 0, 'items_lru'))
        self.assertIsNone(uwsgi.cache_get('KEY1', 'items_lru'))
        uwsgi.cache_get('KEY3', 'items_lru')
        for i in range(4, 100):
            self.assertTrue(uwsgi.cache_set('KEY%d' % i, 'Y' * 20, 0, 'items_lru'))
            self.assertIsNone(uwsgi.cache_get('KEY%d' % (i-2), 'items_lru'))

unittest.main()




import uwsgi


def application(e, s):

    node = uwsgi.cluster_best_node()
    print(node)

    if not node:
        print("sorry node unavailable")
        raise StopIteration

    for part in uwsgi.send_message(node, 0, 0, e, 0, e['wsgi.input'].fileno(), uwsgi.cl()):
        yield part



import os
NAME='glusterfs'

CFLAGS = os.popen('pkg-config --cflags glusterfs-api').read().rstrip().split()
LDFLAGS = []
LIBS = os.popen('pkg-config --libs glusterfs-api').read().rstrip().split() 
GCC_LIST = ['glusterfs']



from threading import Thread
import time
import uwsgi


def mess():
    while True:
        for i in range(0, 100):
            if uwsgi.ready():
                uwsgi.signal(17)
            print(i)
            time.sleep(0.1)

t = Thread(target=mess)
t.daemon = True
t.start()

print("thread started")



from six.moves import range

def mygen(uri):
    for i in range(1, 100):
        yield "ciao %s<br/>" % uri


def application(env, start_response=None):
    return '200 OK',  [('Content-Type', 'text/html')], "<h1>This is the fastest homepage of the world !!!</h1>"
    # return '200 OK',  [('Content-Type', 'text/html')], mygen(env['PATH_INFO'])



import uwsgi

uwsgi.log("I am uWSGI %s" % uwsgi.version)


def application(env, start_response):
    start_response('200 OK', [('Content-Type', 'text/html')])
    uwsgi.log(str(env))

    if env['PATH_INFO'] == '/logme':
        uwsgi.log_this_request()
    return "log written"



import sys
import mimetypes

basedir = sys.argv[1]
mimetypes.init()


def application(environ, start_response):

    filename = basedir + environ['PATH_INFO']
    (content_type, encoding) = mimetypes.guess_type(filename)
    if not content_type:
        content_type = 'text/plain'

    start_response('200 OK', [('Content-Type', content_type)])
    fd = open(filename)
    yield environ['wsgi.file_wrapper'](fd, 32*1024)



import socket
import errno
import struct


def send_request(env, client):

    client.setblocking(1)

    data = "hello world\r\n"

    # send uwsgi-echo header
    client.send(struct.pack('<BHB', 101, len(data), 0))

    # send body
    client.send(data)

    while 1:
        yield env['x-wsgiorg.fdevent.readable'](client.fileno(), 10)
        buf = client.recv(4096)
        if len(buf) == 0:
            break
        else:
            yield buf


def application(env, start_response):

    s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    s.setblocking(0)

    # env['x-wsgiorg.fdevent.readable'] = lambda fd,t: ""
    # env['x-wsgiorg.fdevent.writable'] = lambda fd,t: ""

    # yield ""

    c = s.connect_ex('echo.sock')
    if c == errno.EINPROGRESS:
        yield env['x-wsgiorg.fdevent.writable'](s.fileno(), 10)
        for r in send_request(env, s):
            yield r
    elif c == errno.EISCONN or c == 0:
        start_response('200 Ok', [('Content-Type', 'text/plain')])
        for r in send_request(env, s):
            yield r
    else:
        print(c)
        start_response('500 Internal Server Error', [('Content-Type', 'text/plain')])
        yield "Internal Server Error"

    s.close()



import time


def application(env, start_response):
    start_response('200 OK', [('Content-Type', 'text/html')])
    for i in range(1, 1000):
        yield "<h1>%s at %s</h1>" % (i, str(time.time()))



import werkzeug
import uwsgi


def zero_app(e, s):
    s('200 OK', [('Content-Type', 'text/plain')])
    return "0"


def not_found(e, s):
    s('404 Not Found', [('Content-Type', 'text/plain')])
    return ""


def wsgi_app(e, s):
    s('200 OK', [('Content-Type', 'text/plain')])
    yield "I am a WSGI app\n"


def internal_server_error(e, s):
    s('500 Internal Server Error', [('Content-Type', 'text/plain')])
    return ""

uwsgi.applications = {
    '/wsgi': wsgi_app,
    '/test': werkzeug.test_app,
    '/zero': zero_app,
    '/notfound': not_found,
    '/ise': internal_server_error
}



#!./uwsgi --http-socket :9090 --gevent 100 --module tests.websockets_chat --gevent-monkey-patch
import uwsgi
import time
import gevent.select
import redis


def application(env, sr):

    ws_scheme = 'ws'
    if 'HTTPS' in env or env['wsgi.url_scheme'] == 'https':
        ws_scheme = 'wss'

    if env['PATH_INFO'] == '/':
        sr('200 OK', [('Content-Type', 'text/html')])
        return """
    <html>
      <head>
          <script language="Javascript">
            var s = new WebSocket("%s://%s/foobar/");
            s.onopen = function() {
              alert("connected !!!");
              s.send("ciao");
            };
            s.onmessage = function(e) {
                var bb = document.getElementById('blackboard')
                var html = bb.innerHTML;
                bb.innerHTML = html + '<br/>' + e.data;
            };

            s.onerror = function(e) {
                        alert(e);
                }

        s.onclose = function(e) {
                alert("connection closed");
        }

            function invia() {
              var value = document.getElementById('testo').value;
              s.send(value);
            }
          </script>
     </head>
    <body>
        <h1>WebSocket</h1>
        <input type="text" id="testo"/>
        <input type="button" value="invia" onClick="invia();"/>
        <div id="blackboard" style="width:640px;height:480px;background-color:black;color:white;border: solid 2px red;overflow:auto">
        </div>
    </body>
    </html>
        """ % (ws_scheme, env['HTTP_HOST'])
    elif env['PATH_INFO'] == '/favicon.ico':
        return ""
    elif env['PATH_INFO'] == '/foobar/':
        uwsgi.websocket_handshake(env['HTTP_SEC_WEBSOCKET_KEY'], env.get('HTTP_ORIGIN', ''))
        print("websockets...")
        r = redis.StrictRedis(host='localhost', port=6379, db=0)
        channel = r.pubsub()
        channel.subscribe('foobar')

        websocket_fd = uwsgi.connection_fd()
        redis_fd = channel.connection._sock.fileno()

        while True:
            # wait max 4 seconds to allow ping to be sent
            ready = gevent.select.select([websocket_fd, redis_fd], [], [], 4.0)
            # send ping on timeout
            if not ready[0]:
                uwsgi.websocket_recv_nb()
            for fd in ready[0]:
                if fd == websocket_fd:
                    msg = uwsgi.websocket_recv_nb()
                    if msg:
                        r.publish('foobar', msg)
                elif fd == redis_fd:
                    msg = channel.parse_response()
                    # only interested in user messages
                    if msg[0] == 'message':
                        uwsgi.websocket_send("[%s] %s" % (time.time(), msg))



import uwsgi

import random
import string

items = {}


def gen_rand_n(max_n):
    return random.randint(8, max_n)


def gen_rand_s(size):
    return ''.join(random.choice(string.letters) for i in range(size))

print('filling cache...')
for i in range(0, 1000):
    kl = gen_rand_n(200)
    key = gen_rand_s(kl)
    vl = gen_rand_n(10000)
    val = gen_rand_s(vl)
    items[key] = val
    uwsgi.cache_set(key, val)

print('checking cache...')
count = 0
for key in items.keys():
    val = uwsgi.cache_get(key)
    count += 1
    if val != items[key]:
        print(len(val), val)
        print(len(items[key]), items[key])
        raise Exception('CACHE TEST FAILED AFTER %d ITERATIONS !!!' % count)

print("TEST PASSED")



import uwsgi

sleepvalue = 5


def application(env, start_response):
    start_response('200 Ok', [('Content-type', 'text/html')])
    yield uwsgi.async_sleep(sleepvalue)
    # print("TIMEOUT: ", env['x-wsgiorg.fdevent.timeout'])
    yield "<h1>Hello World after %d seconds</h1>" % sleepvalue



import os
import shutil

jvm_path = 'plugins/jvm'

up = {}
try:
    execfile('%s/uwsgiplugin.py' % jvm_path, up)
except:
    f = open('%s/uwsgiplugin.py' % jvm_path)
    exec(f.read(), up)
    f.close()

NAME='ring'
CFLAGS = up['CFLAGS']
CFLAGS.append('-I%s' % jvm_path)
LDFLAGS = []
LIBS = []
GCC_LIST = ['ring_plugin']

def post_build(config):
    env = os.environ.get('VIRTUAL_ENV')
    if env:
        plugin = "%s/ring_plugin.so" % os.getcwd()
        if os.path.exists(plugin):
            tgt = "%s/bin/ring_plugin.so" % env
            shutil.copyfile(plugin, tgt)
            print("*** ring_plugin.so had been copied to %s" % tgt)




import uwsgi
import os

from os.path import abspath, dirname, join

logo_png = abspath(join(dirname(__file__), "../logo_uWSGI.png"))


def xsendfile(e, sr):
    sr(
        "200 OK", [("Content-Type", "image/png"), ("X-Sendfile", logo_png),],
    )
    return b""


def serve_logo(e, sr):
    sr("200 OK", [("Content-Type", "image/png")])
    return uwsgi.sendfile(logo_png)


def serve_config(e, sr):
    sr("200 OK", [("Content-Type", "text/html")])
    for opt in uwsgi.opt.keys():

        def decode_if_bytes(val):
            if isinstance(val, bytes):
                return val.decode("ascii")
            return val

        body = "{opt} = {optvalue}<br/>".format(
            opt=opt.decode("ascii"), optvalue=decode_if_bytes(uwsgi.opt[opt])
        )
        yield bytes(body.encode("ascii"))


routes = {}
routes["/xsendfile"] = xsendfile
routes["/logo"] = serve_logo
routes["/config"] = serve_config


def application(env, start_response):

    if env["PATH_INFO"] in routes:
        return routes[env["PATH_INFO"]](env, start_response)

    start_response("200 OK", [("Content-Type", "text/html")])

    body = """
<img src="/logo"/> version {version}<br/>
<hr size="1"/>

Configuration<br/>
<iframe src="/config"></iframe><br/>

<br/>

    """.format(
        version=uwsgi.version
    )

    return [bytes(body.encode("ascii"))]



def application(environ, start_response):
    print(environ)
    start_response('200 OK', [])
    if not environ['wsgi.input_terminated']:
        return []
#    print(environ['wsgi.input'].read())
    data = environ['wsgi.input'].read(2)
    print(data)
    data = environ['wsgi.input'].read(2)
    print(data)
    data = environ['wsgi.input'].read(2)
    print(data)
    data = environ['wsgi.input'].read(6)
    print(data)
    print(environ['wsgi.input'].read())
    return [data]



import uwsgi
import time


from six.moves import range


def application(env, start_response):

    start_response('200 Ok', [('Content-Type', 'text/html')])

    yield "I am the worker %d<br/>" % uwsgi.worker_id()

    grunt = uwsgi.grunt()

    if grunt is None:
        print("worker %d detached" % uwsgi.worker_id())
    else:
        yield "And now i am the grunt with a fix worker id of %d<br/>" % uwsgi.worker_id()
        time.sleep(2)
        yield "Now, i will start a very slow task...<br/>"
        for i in range(1, 10):
            yield "waiting for %d seconds<br/>" % i
            time.sleep(i)



import uwsgi
import time


def application(env, start_response):
    counter = 0
    start_response('200 OK', [('Content-Type', 'text/html')])
    start_time = time.time()
    for i in range(1, 100000):
        uwsgi.suspend()
        # print every 100
        if i % 100 == 0:
            yield "<h1>%d</h1>\n" % i
        counter = counter + i

    yield "<h1>%d cycles after %d</h1>\n" % (counter, time.time() - start_time)



import requests

headers = {'Content-Type': 'application/octet-stream'}
data = '\n'.join(['{:04}'.format(i) for i in range(1001)] + ['final'])

r = requests.post("http://127.0.0.1:8000", data=data, headers=headers)

assert r.text == data



def app(req):
    print(req)
    ret = {
        "status": 200,
        "headers": {
            "content_type": "text/html",
            "foo": ['bar0', 'bar1', 'bar2']
        },
        "body": "<h1>Hello!</h1>",
    }
    print(ret)
    return ret



import uwsgi
import os

print("!!! uWSGI version:", uwsgi.version)


def ciao():
    print("modifica su /tmp")


def ciao2():
    print("nuovo uwsgi_server")
    print(os.getpid())

counter = 0

#if uwsgi.load_plugin(0, 'plugins/example/example_plugin.so', 'ciao'):
#    print("example plugin loaded")
#else:
#    print("unable to load example plugin")

#uwsgi.event_add(uwsgi.EVENT_FILE, "/tmp", ciao)
#uwsgi.event_add(uwsgi.EVENT_DNSSD, "_uwsgi._tcp", ciao2)
#uwsgi.event_add(uwsgi.EVENT_TIMER, 1000, ciao2)

uwsgi.post_fork_hook = ciao2


def application(env, start_response):
    global counter

    start_response('200 Ok', [('Content-type', 'text/plain')])
    yield "hello world"
    yield "hello world2"

    for i in range(1, 1000):
        yield str(i)

    yield "\n"

    yield str(counter)
    counter += 1



import uwsgi
import unittest

class SharedareaTest(unittest.TestCase):

    def setUp(self):
        uwsgi.sharedarea_write(0, 0, '\0' * 64)

    def test_32(self):
        uwsgi.sharedarea_write32(0, 0, 17)
        self.assertEqual(uwsgi.sharedarea_read32(0, 0), 17)

    def test_inc32(self):
        uwsgi.sharedarea_write32(0, 4, 30)
        uwsgi.sharedarea_inc32(0, 4, 3)
        self.assertEqual(uwsgi.sharedarea_read32(0, 4), 33)

    def test_dec32(self):
        uwsgi.sharedarea_write32(0, 5, 30)
        uwsgi.sharedarea_dec32(0, 5, 4)
        self.assertEqual(uwsgi.sharedarea_read32(0, 5), 26)

    def test_inc64(self):
        uwsgi.sharedarea_write64(0, 8, 17 * (1024 ** 5))
        uwsgi.sharedarea_inc64(0, 8, 1)
        self.assertEqual(uwsgi.sharedarea_read64(0, 8), 17 * (1024 ** 5) + 1)

    def test_dec64(self):
        uwsgi.sharedarea_write64(0, 8, 30 * (1024 ** 5))
        uwsgi.sharedarea_dec64(0, 8, 30 * (1024 ** 5) - 1)
        self.assertEqual(uwsgi.sharedarea_read64(0, 8), 1)
    

unittest.main()



import uwsgi
import threading
import time


def foo():
    while True:
        time.sleep(1)
        print("ciao, sono un thread")

t = threading.Thread(target=foo)
t.daemon = True
t.start()


def application(e, s):
    s('200 OK', [('Content-Type', 'text/html')])
    for i in range(0, 3):
        yield uwsgi.async_sleep(1)
        yield "iter: %d<br/>" % i



from picasso import *


def home(req):
    if not req["session"].get("user"):
        return redirect("/login")
    return "<h1>Welcome back, %s!</h1>" % req["session"]["user"]


def login(req):
    return "<form method='post'><input type='submit' name='foo' value='login' /></form>"


def login_post(req):
    print(req)
    req["session"]["user"] = "James"
    return redirect("/")

routes = setup_routes(
    GET("/", home),
    GET("/login", login),
    POST("/login", login_post),
    routing.not_found("<h1>Not Found</h1>")
)

app = setup_app(routes)



import uwsgi

from mako.template import Template
import time


def application(env, start_response):
    start_response('200 OK', [('Content-Type', 'text/html')])

    mytemplate = Template("<h1>I am Mako at ${thetime}</h1>")
    uwsgi.green_schedule()

    yield mytemplate.render(thetime=time.time())

    for i in range(1, 100):
        mytemplate = Template("Iteration ${thei} at ${thetime}<br/>")
        uwsgi.green_schedule()
        yield mytemplate.render(thei=i, thetime=time.time())



import uwsgi
import unittest

class SharedareaTest(unittest.TestCase):

    def test_32(self):
        pos = 2 * (1024 ** 3)
        uwsgi.sharedarea_write32(0, pos, 17)
        self.assertEqual(uwsgi.sharedarea_read32(0, pos), 17)

    def test_64(self):
        pos = 2 * (1024 ** 3)
        uwsgi.sharedarea_write64(0, pos, 30)
        self.assertEqual(uwsgi.sharedarea_read64(0, pos), 30)

unittest.main()



# uwsgi -s :3031 -M -p 4 --plugin gevent --loop gevent --async 1000 --enable-threads -w tests.testgevent
from threading import Thread
import gevent
import uwsgi
import time


def microtask(wid):
    print("i am a gevent task")
    gevent.sleep(10)
    print("10 seconds elapsed in worker id %d" % wid)


def athread():
    while True:
        time.sleep(1)
        print("i am the thread 1")


def athread2():
    while True:
        time.sleep(1)
        print("i am the thread 2")

t1 = Thread(target=athread)
t1.daemon = True
t1.start()

t2 = Thread(target=athread2)
t2.daemon = True
t2.start()


def application(environ, start_response):

    gevent.sleep()
    start_response('200 OK', [('Content-Type', 'text/html')])
    yield "sleeping for 3 seconds...<br/>"
    gevent.sleep(3)
    yield "done<br/>"
    gevent.spawn(microtask, uwsgi.worker_id())
    yield "microtask started<br/>"



# See spooler_decorator_tests

from uwsgidecorators import spool, spoolraw
import uwsgi

ghostpath = "/tmp/ghost"


@spool(pass_arguments=True)
def controlled_arguments_task(*args, **kwargs):
    if args != ({'key': 'value'}, 2) or kwargs != {'key1': 'value1'}:
        print("We have a problem!")
        open(ghostpath, 'w').close()
    uwsgi.signal(20)


@spool
def controlled_task(arguments):
    if arguments['arg'] != 'alive' and 'ghost' in arguments:
        print("We have a problem!")
        open(ghostpath, 'w').close()
    uwsgi.signal(20)


@spoolraw
def controlled_raw_task(arguments):
    if arguments['arg'] != 'alive' and 'ghost' in arguments:
        print("We have a problem!")
        open(ghostpath, 'w').close()
    uwsgi.signal(20)
    return uwsgi.SPOOL_OK



import time
import stackless


def application(env, start_response):
    start_response('200 OK', [('Content-Type', 'text/html')])

    for i in range(1, 100000):
        yield "<h1>%s at %s</h1>\n" % (i, str(time.time()))
        # schedule every 2
        if i % 2 == 0:
            stackless.schedule()

    print("DONE AT %d" % i)



"""
simplified test for continulet without checking for partial writes

to enable continulets you only need to call uwsgi_pypy_setup_continulets() soon after startup:

uwsgi --pypy-wsgi-file t/pypy/t_continulet1.py --http-socket :9090 --pypy-home /opt/pypy --pypy-eval "uwsgi_pypy_setup_continulets()" --async 8

"""
import uwsgi


def application(e, sr):
    sr('200 OK', [('Content-Type', 'text/plain')])

    # call suspend 10 times and yield some value
    for i in range(0, 10):
        print(i)
        uwsgi.suspend()
        yield str(i)

    # connect to a memcached server
    fd = uwsgi.async_connect('127.0.0.1:11211')
    try:
        # start waiting for socket availability (4 seconds max)
        uwsgi.wait_fd_write(fd, 4)
        # suspend execution 'til event
        uwsgi.suspend()
        uwsgi.send(fd, "get /foobar\r\n")
        # now wait for memcached response
        uwsgi.wait_fd_read(fd, 4)
        uwsgi.suspend()
        # read the response
        data = uwsgi.recv(fd, 4096)
        # return to the client
        yield data
    finally:
        uwsgi.close(fd)

    print("sleeping for 3 seconds...")
    uwsgi.async_sleep(3)
    uwsgi.suspend()
    yield "done"



import uwsgi
import struct
import sys

print(sys.argv)
if len(sys.argv) == 3:
    chunks = uwsgi.send_message(sys.argv[1], 10, int(sys.argv[2]), '')

    pkt = ''

    for chunk in chunks:
        pkt += chunk

    print("%d = %d" % (int(sys.argv[2]), struct.unpack("I", pkt)[0]))
elif len(sys.argv) == 4:
    uwsgi.send_message(sys.argv[1], 10, int(sys.argv[2]), struct.pack("I", int(sys.argv[3])))



import uwsgi
from uwsgidecorators import *


@signal(17, target='workers')
def hello(signum):
    print("I AM THE WORKER %d" % uwsgi.worker_id())


@signal(30, target='worker2')
def hello2(signum):
    print("I AM THE WORKER 2")


@postfork
def wait_for_signal():
    if uwsgi.worker_id() != 2:
        print("waiting for a signal...")
        uwsgi.signal_wait()
        print("signal %d received" % uwsgi.signal_received())


def application(e, s):
    s('200 OK', [('Content-Type', 'text/html')])
    if e['PATH_INFO'] == '/30':
        uwsgi.signal(30)
        uwsgi.signal(100)
    else:
        uwsgi.signal(17)
    return "Signal raised"



from uwsgidecorators import *


@timer(3, target='mule1')
def hello_timer(signum):
    print("3 seconds elapsed")


@mulefunc
def conto_fino_a_dieci(uno, due, tre):
    print("MULE ID %d: conto_fino_a_dieci" % uwsgi.mule_id(), uno, due, tre)


@mulefunc(2)
def conto_fino_a_venti(uno, due, tre):
    print("MULE ID %d: conto_fino_a_venti" % uwsgi.mule_id(), uno, due, tre)


@mulefunc('topogigio')
def conto_fino_a_trenta(uno, due, tre):
    print("MULE ID %d: conto_fino_a_trenta" % uwsgi.mule_id(), uno, due, tre)


def application(e, sr):
    conto_fino_a_dieci(1, 2, 3)
    conto_fino_a_venti(4, 5, 6)
    conto_fino_a_trenta(7, 8, 9)
    sr('200 OK', [('Content-Type', 'text/html')])
    return "MULE !!!"



import uwsgi
import time


def application(env, start_response):
    sleepvalue = 5
    if 'QUERY_STRING' in env:
        if env['QUERY_STRING'] != '':
            sleepvalue = int(env['QUERY_STRING'])
        start_response('200 Ok', [('Content-type', 'text/html')])
    start_at = time.time()
    uwsgi.green_sleep(sleepvalue)
    # print("TIMEOUT: ", env['x-wsgiorg.fdevent.timeout'])
    yield "<h1>Hello World after %s seconds</h1>" % str(time.time() - start_at)
